#pragma once

#include <memory>
#include <thread>
#include <mutex>
#include <condition_variable>

#include "service.h"

/*
 * Service class for maintaining a pump
 */
class CPump final : public IService
{
	private:
		// pump runner thread
		std::unique_ptr<std::thread> mPump_Thread;

		// shared mutex
		std::mutex mRun_Mtx;
		// shared CV
		std::condition_variable mRun_Cv;

		// pump identifier
		size_t mPump_Id = 0;
		// is running?
		bool mRunning = false;

		// current rate (PWM stride, 0-1024)
		int mCurrent_Rate = 0;

		// currently used PWM pin for given pump
		int mPwm_Pin = 0;

	protected:
		// pump control thread method
		void Thread_Fnc();

	public:
		CPump();
		~CPump();

		// retrieves current pump rate (PWM stride, 0-1024)
		int Get_Current_Rate() const;
		// sets the current pump rate (PWM stride, 0-1024)
		void Set_Current_Rate(int rate);

		// IService interface
		virtual bool Start(size_t nParam) override;
		virtual bool Stop() override;

		// INamed_Service concept
		constexpr static const char* Service_Name = "Pump";
};
