#pragma once

#include <memory>
#include <thread>
#include <mutex>
#include <condition_variable>

#include "service.h"

/*
 * Service class for maintaining sensor readings
 */
class CSensor final : public IService
{
	private:
		// sensor control thread
		std::unique_ptr<std::thread> mSensor_Thread;

		// shared mutex
		std::mutex mRun_Mtx;
		// shared CV
		std::condition_variable mRun_Cv;
		// shared CV for measurement awaiters
		std::condition_variable mValue_Await_Cv;

		// sensor identifier
		size_t mSensor_Id = 0;
		// is running?
		bool mRunning = false;

		// current sensor reading
		double mCurrent_Value = 0;
		// current sensor reading timestamp (Lamport-like)
		size_t mCurrent_Timestamp = 1;

		// currently used trigger pin
		int mTrig_Pin = 0;
		// currently used echo pin
		int mEcho_Pin = 0;

		// is measurement triggered? (should we signalize the CV?)
		bool mMeasurement_Triggered = false;

	protected:
		// sensor control thread method
		void Thread_Fnc();

	public:
		CSensor();
		~CSensor();

		// trigger measurement
		void Trigger_Measurement();
		// trigger measurement and block, until the measurement value is obtained
		double Trigger_And_Await_Measurement();

		// get current sensor reading
		double Get_Sensor_Value() const;
		// get current sensor reading timestamp (Lamport-like, non-decreasing)
		size_t Get_Timestamp() const;

		// IService interface
		virtual bool Start(size_t nParam) override;
		virtual bool Stop() override;

		// INamed_Service concept
		constexpr static const char* Service_Name = "Sensor";
};
