#pragma once

/*
 * This is a file that maintains a bridge between wiringPi and application code
 * 
 * We need to mock pin states on development platforms, as we do not have equal environments and also need to
 * perform some structured testing without the real hardware.
 */

#include <chrono>

// available pin modes (maps to wiringPi modes)
enum class PinModes : int {
	_INPUT = 0,
	_OUTPUT = 1,
	_PWM_OUTPUT = 2,
};

// converts PinModes enumerator to integer
inline constexpr int PinMode(PinModes mode) {
	return static_cast<int>(mode);
}

// available pin levels (maps to wiringPi levels)
enum class PinLevels : int {
	_LOW = 0,
	_HIGH = 1,
};

// converts PinLevels enumerator to integer
inline constexpr int PinLevel(PinLevels lvl) {
	return static_cast<int>(lvl);
}

#if defined(MOCKED_PINS)

// specify needed PWM modes
#ifndef PWM_MODE_MS
#define PWM_MODE_MS 0
#endif

// wiringPi interface functions 1:1

int wiringPiSetup();
void pinMode(int pinNo, int mode);
int digitalRead(int pinNo);
void digitalWrite(int pinNo, int level);
void pwmWrite(int pinNo, int dutyCycle);
void pwmSetClock(int);
void pwmSetMode(int);

#else // defined(MOCKED_PINS)

#ifdef TRUE
#undef TRUE
#undef FALSE
#endif

#include <wiringPi.h>
#endif // defined(MOCKED_PINS)

// mocking functions namespace
namespace mock {
	void pinState(int pinNo, PinLevels level);
	void pinStateFor(int pinNo, PinLevels levelNow, PinLevels levelLater, std::chrono::high_resolution_clock::duration dur);
}
