var backendWs;

var glucData = [];
var carbData = [];
var insData = [];
var xValues = [];
var chart = null;
var sensor_timestamp = 0;

function updatePlot() {
    if (chart == null)
    {
        Chart.defaults.font.size = 16;
        chart = new Chart("glucChart", {
            type: "line",
            data: {
              labels: xValues,
              datasets: [{
                label: "Interstitial glucose",
                data: glucData,
                borderColor: "blue",
                fill: false
              }, {
                label: "Carbohydrates",
                data: carbData,
                borderColor: "green",
                showLine: false,
                backgroundColor: "none",
                pointRadius: 10,
                pointBackgroundColor: "green",
                fill: false
              }, {
                label: "Insulin",
                data: insData,
                borderColor: "fuchsia",
                showLine: false,
                backgroundColor: "none",
                pointRadius: 10,
                pointBackgroundColor: "fuchsia",
                fill: false
              }]
            },
            options: {
                plugins: {
                    legend: {
                        display: false
                    },
                },
              animation: {
                duration: 0
              },
              scales: {
                y: {
                    min: 2.0,
                    max: 20.0
                }
              }
            }
          });
    }
    else
    {
        chart.data.datasets.pop();
        chart.data.datasets.pop();
        chart.data.datasets.pop();

        chart.data.datasets.push({
            label: "Interstitial glucose",
            data: glucData,
            borderColor: "blue",
            fill: false
        });
        chart.data.datasets.push({
            label: "Carbohydrates",
            data: carbData,
            borderColor: "green",
            showLine: false,
            backgroundColor: "none",
            pointBackgroundColor: "green",
            pointRadius: 10,
            fill: false
        });
        chart.data.datasets.push({
            label: "Insulin",
            data: insData,
            borderColor: "fuchsia",
            showLine: false,
            backgroundColor: "none",
            pointRadius: 10,
            pointBackgroundColor: "fuchsia",
            fill: false
          });
        chart.data.labels = xValues;
        chart.update();
    }
}

function connectToWS() {

    $('.conn_indicator').removeClass("online").removeClass("offline").addClass("offline");
    $('.conn_desc').text("Connecting...");

    var endpoint = "ws://localhost:9002";
    if (backendWs !== undefined && backendWs.readyState === WebSocket.OPEN) {
        return;
    }

    backendWs = new WebSocket(endpoint);

    backendWs.onmessage = function(event) {
        var leng;
        if (event.data.size === undefined) {
            leng = event.data.length
        } else {
            leng = event.data.size
        }
        console.log("Message received. Size: " + leng + ", content: " + event.data);

        var str = String(event.data);
        if (str.startsWith('sensor-value')) {
            var sides = str.split("=");
            if (sides.length == 3) {
                try {
                    var ts = parseInt(sides[2]);
                    sensor_timestamp = ts;

                    var fl = parseFloat(sides[1]);
                    glucData.push(fl);
                    carbData.push(0);
                    insData.push(0);

                    while (glucData.length > 108) {
                        glucData.shift();
                        carbData.shift();
                        insData.shift();
                    }

                    while (xValues.length < glucData.length)
                        xValues.unshift("-"+(xValues.length*5)+" mins");
                    updatePlot();

                    $('.cur_glucose').text(fl.toFixed(1));
                }
                catch {
                    //
                }
            }
        }
        else if (str.startsWith('pump-basal')) {
            var sides = str.split("=");
            if (sides.length == 2) {
                try {
                    var fl = parseFloat(sides[1]);
                    $('.cur_basal').text(fl.toFixed(1));
                }
                catch {
                    //
                }
            }
        }
        else if (str.startsWith('wp')) {
            var sides = str.substr(3, 1024).split(",");
            if (sides.length == 2) {
                try {
                    var p0 = Math.ceil(100 * parseInt(sides[0]) / 1024.0);
                    var p1 = Math.ceil(100 * parseInt(sides[1]) / 1024.0);

                    console.log(p0);
                    console.log(p1);

                    $('#pump0rate').css("width", p0 + "%");
                    $('#pump1rate').css("width", p1 + "%");
                }
                catch {
                    //
                }
            }
        }
    }

    backendWs.onopen = function(evt) {
        console.log("Connection opened!");
        $('.conn_indicator').removeClass("offline").addClass('online');
        $('.conn_desc').text("Connected");
    };

    backendWs.onclose = function(evt) {
        console.log("Closed");
        glucData = [];
        xValues = [];
        sensor_timestamp = 0;
        $('.conn_indicator').removeClass("online").addClass('offline');
        $('.conn_desc').text("Disconnected");

        setTimeout(connectToWS, 2000);
    };

    backendWs.onerror = function(evt) {
        console.log("Error! Reconnecting");
        $('.conn_indicator').removeClass("online").addClass('offline');
        $('.conn_desc').text("Error!");

        backendWs.close();
    };
}

function updateBackend() {

    if (typeof backendWs === 'undefined' || backendWs == null || backendWs.readyState !== WebSocket.OPEN)
        return;

    backendWs.send("read-sensor="+(sensor_timestamp));
    backendWs.send("read-pump-basal");
    backendWs.send("read-wp-params");
}

function sendMeal(carbs) {

    if (typeof backendWs === 'undefined' || backendWs == null || backendWs.readyState !== WebSocket.OPEN)
        return;

    backendWs.send("eat-carbs=" + carbs);

	var cdata = carbs/10;
	
    carbData.push(cdata < 2.5 ? 2.5 : cdata);
    carbData.shift();
    updatePlot();
}

function sendBasal(basal) {

    if (typeof backendWs === 'undefined' || backendWs == null || backendWs.readyState !== WebSocket.OPEN)
        return;

    backendWs.send("set-basal=" + basal);
}

function sendBolus(bolus) {

    if (typeof backendWs === 'undefined' || backendWs == null || backendWs.readyState !== WebSocket.OPEN)
        return;

    backendWs.send("dose-bolus=" + bolus);

    insData.push(bolus < 2.5 ? 2.5 : bolus);
    insData.shift();
    updatePlot();
}
